@@ -61,6 +61,12 @@ last.qp <- vertical.qp("last.points")
 ### Label first points from QP solver that ensures labels do not collide.
 first.qp <- vertical.qp("first.points")
 
+### Draw a speech polygon to the first point.
+first.polygons <- polygon.method("first.points", -0.1, "right", "left")
+
+### Draw a speech polygon to the last point.
+last.polygons <- polygon.method("last.points", 0.1, "left", "right")
+
 ### Label first or last points, whichever are more spread out, and use
 ### a QP solver to make sure the labels do not collide.
 maxvar.qp <- vertical.qp("maxvar.points")
@@ -106,3 +112,8 @@ lines2 <- function
     transform(pos,vjust=if(bigger.on.average)top else bottom)
   })
 }
+
+### Draw a box with the label inside, at the point furthest away from
+### the plot border and any other curve.
+angled.boxes <-
+  list("far.from.others.borders","calc.boxes","enlarge.box","draw.rects")
